@@ -7,6 +7,7 @@ import (
 	"strings"
 	"time"
 
+	"github.com/go-openapi/swag"
 	"github.com/google/uuid"
 	"github.com/pkg/errors"
 	"github.com/sirupsen/logrus"
@@ -26,7 +27,6 @@ import (
 
 const (
 	InstallDir                   = "/opt/install-dir"
-	KubeconfigPathLoopBack       = "/opt/openshift/auth/kubeconfig-loopback"
 	KubeconfigPath               = "/opt/openshift/auth/kubeconfig"
 	minMasterNodes               = 2
 	dockerConfigFile             = "/root/.docker/config.json"
@@ -85,15 +85,16 @@ func (i *installer) InstallNode() error {
 		return err
 	}
 	ctx, cancel := context.WithCancel(context.Background())
-	errs, _ := errgroup.WithContext(ctx)
+	bootstrapErrGroup, _ := errgroup.WithContext(ctx)
 	//cancel the context in case this method ends
 	defer cancel()
 	isBootstrap := false
 	if i.Config.Role == string(models.HostRoleBootstrap) && i.HighAvailabilityMode != models.ClusterHighAvailabilityModeNone {
 		isBootstrap = true
-		errs.Go(func() error {
-			return i.runBootstrap(ctx)
+		bootstrapErrGroup.Go(func() error {
+			return i.startBootstrap()
 		})
+		go i.updateConfiguringStatus(ctx)
 		i.Config.Role = string(models.HostRoleMaster)
 	}
 
@@ -124,13 +125,23 @@ func (i *installer) InstallNode() error {
 	}
 
 	if isBootstrap {
-		i.UpdateHostInstallProgress(models.HostStageWaitingForControlPlane, "")
-		if err = errs.Wait(); err != nil {
-			i.log.Error(err)
+		if err = bootstrapErrGroup.Wait(); err != nil {
+			i.log.Errorf("Bootstrap failed %s", err)
+			return err
+		}
+		if err = i.waitForControlPlane(ctx); err != nil {
+			return err
+		}
+		i.log.Info("Setting bootstrap node new role to master")
+
+	} else if i.Config.Role == string(models.HostRoleWorker) {
+		// Wait for 2 masters to be ready before rebooting
+		if err = i.workerWaitFor2ReadyMasters(ctx); err != nil {
 			return err
 		}
 	}
 	//upload host logs and report log status before reboot
+	i.log.Infof("Uploading logs and reporting status before rebooting the node %s for cluster %s", i.Config.HostID, i.Config.ClusterID)
 	i.inventoryClient.HostLogProgressReport(ctx, i.Config.ClusterID, i.Config.HostID, models.LogsStateRequested)
 	_, err = i.ops.UploadInstallationLogs(isBootstrap || i.HighAvailabilityMode == models.ClusterHighAvailabilityModeNone)
 	if err != nil {
@@ -188,32 +199,14 @@ func (i *installer) writeImageToDisk(ignitionPath string) error {
 	return nil
 }
 
-func (i *installer) runBootstrap(ctx context.Context) error {
-	ctxLocal, cancel := context.WithCancel(ctx)
-	defer func() {
-		cancel()
-	}()
-	go i.updateConfiguringStatus(ctxLocal)
-
-	err := i.startBootstrap()
-	if err != nil {
-		i.log.Errorf("Bootstrap failed %s", err)
-		return err
-	}
-	kc, err := i.kcBuilder(KubeconfigPathLoopBack, i.log)
-	if err != nil {
-		i.log.Error(err)
-		return err
-	}
-	if err = i.waitForControlPlane(ctx, kc); err != nil {
-		return err
-	}
-	i.log.Info("Setting bootstrap node new role to master")
-	return nil
-}
-
 func (i *installer) startBootstrap() error {
 	i.log.Infof("Running bootstrap")
+	// This is required for the log collection command to work since it will try to mount this directory
+	// This directory is also required by `generateSshKeyPair` as it will place the key there
+	if err := i.ops.Mkdir(sshDir); err != nil {
+		i.log.WithError(err).Error("Failed to create SSH dir")
+		return err
+	}
 	ignitionFileName := "bootstrap.ign"
 	ignitionPath, err := i.getFileFromService(ignitionFileName)
 	if err != nil {
@@ -308,10 +301,6 @@ func (i *installer) extractIgnitionToFS(ignitionPath string) (err error) {
 
 func (i *installer) generateSshKeyPair() error {
 	i.log.Info("Generating new SSH key pair")
-	if err := i.ops.Mkdir(sshDir); err != nil {
-		i.log.WithError(err).Error("Failed to create SSH dir")
-		return err
-	}
 	if _, err := i.ops.ExecPrivilegeCommand(utils.NewLogWriter(i.log), "ssh-keygen", "-q", "-f", sshKeyPath, "-N", ""); err != nil {
 		i.log.WithError(err).Error("Failed to generate SSH key pair")
 		return err
@@ -355,8 +344,20 @@ func (i *installer) waitForNetworkType(kc k8s_client.K8SClient) error {
 	})
 }
 
-func (i *installer) waitForControlPlane(ctx context.Context, kc k8s_client.K8SClient) error {
-	if err := i.waitForMinMasterNodes(ctx, kc); err != nil {
+func (i *installer) waitForControlPlane(ctx context.Context) error {
+	err := i.ops.ReloadHostFile("/etc/resolv.conf")
+	if err != nil {
+		i.log.WithError(err).Error("Failed to reload resolv.conf")
+		return err
+	}
+	kc, err := i.kcBuilder(KubeconfigPath, i.log)
+	if err != nil {
+		i.log.Error(err)
+		return err
+	}
+	i.UpdateHostInstallProgress(models.HostStageWaitingForControlPlane, "")
+
+	if err = i.waitForMinMasterNodes(ctx, kc); err != nil {
 		return err
 	}
 
@@ -377,7 +378,7 @@ func (i *installer) waitForControlPlane(ctx context.Context, kc k8s_client.K8SCl
 	i.waitForBootkube(ctx)
 
 	// waiting for controller pod to be running
-	if err := i.waitForController(); err != nil {
+	if err := i.waitForController(kc); err != nil {
 		i.log.Error(err)
 		return err
 	}
@@ -385,6 +386,32 @@ func (i *installer) waitForControlPlane(ctx context.Context, kc k8s_client.K8SCl
 	return nil
 }
 
+func numDoneMasters(cluster *models.Cluster) int {
+	numDoneMasters := 0
+	for _, h := range cluster.Hosts {
+		if h.Role == models.HostRoleMaster && h.Progress.CurrentStage == models.HostStageDone {
+			numDoneMasters++
+		}
+	}
+	return numDoneMasters
+}
+
+func (i *installer) workerWaitFor2ReadyMasters(ctx context.Context) error {
+	i.log.Info("Waiting for 2 ready masters")
+	i.UpdateHostInstallProgress(models.HostStageWaitingForControlPlane, "")
+	for {
+		cluster, err := i.inventoryClient.GetCluster(ctx)
+		if err != nil {
+			i.log.WithError(err).Errorf("Getting cluster %s", i.ClusterID)
+			return err
+		}
+		if swag.StringValue(cluster.Kind) == models.ClusterKindAddHostsCluster || numDoneMasters(cluster) >= minMasterNodes {
+			return nil
+		}
+		time.Sleep(generalWaitInterval)
+	}
+}
+
 func (i *installer) shouldControlPlaneReplicasPatchApplied(kc k8s_client.K8SClient) (bool, error) {
 	controlPlanePatchRequired, err := utils.IsVersionLessThan47(i.Config.OpenshiftVersion)
 	if err != nil {
@@ -454,7 +481,7 @@ func (i *installer) UpdateHostInstallProgress(newStage models.HostStage, info st
 	log := utils.RequestIDLogger(ctx, i.log)
 	log.Infof("Updating node installation stage: %s - %s", newStage, info)
 	if i.HostID != "" {
-		if err := i.inventoryClient.UpdateHostInstallProgress(ctx, i.HostID, newStage, info); err != nil {
+		if err := i.inventoryClient.UpdateHostInstallProgress(ctx, i.Config.InfraEnvID, i.Config.HostID, newStage, info); err != nil {
 			log.Errorf("Failed to update node installation stage, %s", err)
 		}
 	}
@@ -462,6 +489,8 @@ func (i *installer) UpdateHostInstallProgress(newStage models.HostStage, info st
 
 func (i *installer) waitForBootkube(ctx context.Context) {
 	i.log.Infof("Waiting for bootkube to complete")
+	i.UpdateHostInstallProgress(models.HostStageWaitingForBootkube, "")
+
 	for {
 		select {
 		case <-ctx.Done():
@@ -480,20 +509,9 @@ func (i *installer) waitForBootkube(ctx context.Context) {
 	}
 }
 
-func (i *installer) waitForController() error {
+func (i *installer) waitForController(kc k8s_client.K8SClient) error {
 	i.log.Infof("Waiting for controller to be ready")
-	i.UpdateHostInstallProgress(models.HostStageWaitingForControlPlane, "waiting for controller pod")
-	err := i.ops.ReloadHostFile("/etc/resolv.conf")
-	if err != nil {
-		i.log.WithError(err).Error("Failed to reload resolv.conf")
-		return err
-	}
-
-	kc, err := i.kcBuilder(KubeconfigPath, i.log)
-	if err != nil {
-		i.log.WithError(err).Errorf("Failed to create kc client from %s", KubeconfigPath)
-		return err
-	}
+	i.UpdateHostInstallProgress(models.HostStageWaitingForController, "waiting for controller pod ready event")
 
 	events := map[string]string{}
 	tickerUploadLogs := time.NewTicker(5 * time.Minute)
@@ -628,7 +646,7 @@ func (i *installer) updateReadyMasters(nodes *v1.NodeList, readyMasters *[]strin
 				return fmt.Errorf("Node %s is not in inventory hosts", node.Name)
 			}
 			ctx = utils.GenerateRequestContext()
-			if err := i.inventoryClient.UpdateHostInstallProgress(ctx, host.Host.ID.String(), models.HostStageJoined, ""); err != nil {
+			if err := i.inventoryClient.UpdateHostInstallProgress(ctx, host.Host.InfraEnvID.String(), host.Host.ID.String(), models.HostStageJoined, ""); err != nil {
 				utils.RequestIDLogger(ctx, i.log).Errorf("Failed to update node installation status, %s", err)
 			}
 		}
